/*
*  Rubrica
*  file: file.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <stdlib.h>
#include <ctype.h>
#include <math.h>

#include "file_util.h"

gboolean start = FALSE;

gboolean rubrica_file_is_start(void)
{
  return start;
}

void rubrica_file_set_start(gboolean state)
{
  start = state;
}

/*  return:
    TRUE  if filename has extension .rub
    FALSE otherwise
*/
gboolean rubrica_file_has_extension(gchar *fn)
{
  gchar *tmp=NULL, *buf;
  gint i, len;

  i = 0;
  buf = g_basename(fn);
  len = strlen(buf);

  while(*buf != '.')
    {
      buf++;
      i++;
            
      if (i == len)     /* end buffer was reached and dot wasn't found */
	return FALSE;	
    }

  tmp = (gchar*) g_malloc(((len - i)*sizeof(gchar))+1);
  strcpy(tmp, buf);
  
  if ((strcmp(tmp,".rub")) == 0)
    {
      g_free(tmp);
      return TRUE;
    }
  else 
    g_free(tmp);
  return FALSE;  
}

void rubrica_file_do_backup(gchar *old, gchar *new)
{
  /* se esiste gi un file con lo stesso nome, crea una copia di backup 
     if exist a file with same name, then do a backup of old file */
  if (g_file_exists(new))
    {
      gchar *ext=NULL, *f=NULL;
      gchar *bfile;
      gint num;

      rubrica_file_split(new, f, ext, &num);

      bfile = g_strdup_printf("%s.%s%d",f,ext,num);
      rubrica_file_do_backup(old, bfile);
      g_free(bfile);
    }
  else
    rename(old, new);  
}

void rubrica_file_split(gchar *file, gchar *f, gchar *ext, gint *n)
{
  gchar *buf, *tmp;
  gint i = 0, num = 0;

  f = g_basename(file);
  tmp = g_strdup(f);
  buf = tmp;
  while(*buf != '.')
    {
      buf++;
      i++;
    }

  if (buf)
    strcpy(ext, buf);

  g_free(tmp);
   
  tmp = ext;
  g_strreverse(tmp);
  i = 0;
  num = 0;
  while (tmp)
    {
      if (isdigit(*tmp))
	{
	  num = num * pow(10,i);
	  i++;
	}	
      tmp++;
    }
  num+=1;
  
  *n = num;
}
