/*
*  Rubrica
*  file: modulo.h
*
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* Modulo rubrica, derivato da gnome-propertybox
*/

#ifndef __MODULO_RUBRICA_H
#define __MODULO_RUBRICA_H

#include <libgnomeui/gnome-dialog.h>
#include <libgnome/gnome-defs.h>
#include "rubrica.h"

BEGIN_GNOME_DECLS

#define MODULO_RUBRICA_TYPE            (modulo_rubrica_get_type())
#define MODULO_RUBRICA(obj)            (GTK_CHECK_CAST((obj), MODULO_RUBRICA_TYPE, ModuloRubrica))
#define MODULO_RUBRICA_CLASS(klass)    (GTK_CHECK_CLASS_CAST((klass), MODULO_RUBRICA_TYPE, ModuloRubricaClass))
#define IS_MODULO_RUBRICA(obj)         (GTK_CHECK_TYPE((obj), MODULO_RUBRICA_TYPE))
#define IS_MODULO_RUBRICA_CLASS(klass) (GTK_CHECK_CLASS_TYPE((klass), MODULO_RUBRICA_TYPE))


typedef enum {
  NUOVO_MODULO = 1,
  MODIFICA_MODULO,
  SCONOSCIUTO = 0
} MODO_MODULO;


typedef struct _info Info;

struct _info {
  gint riga;             /* riga selezionata della clist */
  gint colonna;          /* colonna selezionata della clist */
  gboolean selezionato;  /* flag, c' stata selezione? */
};

typedef struct _ModuloRubrica         ModuloRubrica;
typedef struct _ModuloRubricaClass    ModuloRubricaClass;

struct _ModuloRubrica {
  GnomeDialog dialog;

  /* il notebook */
  GtkWidget *notebook;       

  /* elementi del frame dati */
  /* i toogle button */
  GtkWidget *editabile;
  GtkWidget *cancellabile;

  /* entry */
  GtkWidget *scheda;

  GtkWidget *nome;
  GtkWidget *cognome;
  GtkWidget *professione;

  GtkWidget *via;
  GtkWidget *numero;
  GtkWidget *cap;
  GtkWidget *citta;
  GtkWidget *provincia;
  GtkWidget *stato;
  /* ------------------------------ */


  /* elementi del frame rete */
  /* entry */
  GtkWidget *web;
  GtkWidget *mail;

  /* clist */
  GtkWidget *clist_web;
  GtkWidget *clist_mail;

  /* bottoni web, mail */
  GtkWidget *web_aggiungi;
  GtkWidget *web_modifica;
  GtkWidget *web_cancella;

  GtkWidget *mail_aggiungi;
  GtkWidget *mail_modifica;
  GtkWidget *mail_cancella;
  /* ------------------------------ */
  
  /* elementi del frame telefono */
  /* entry */
  GtkWidget *telefono;  
  
  GSList *gruppo_tipo; // = NULL  -- gruppo a cui appartengono i radio buttons
                       //  una lista dei radio button
                       // mi serve ??

  /* radio buttons */
  GtkWidget *radio_casa;
  GtkWidget *radio_ufficio;
  GtkWidget *radio_fax;
  GtkWidget *radio_cellulare;
  GtkWidget *radio_altro;

  /* bottoni telefono */
  GtkWidget *telefono_aggiungi;
  GtkWidget *telefono_modifica;
  GtkWidget *telefono_cancella;

  /* clist */
  GtkWidget *clist_telefono;
  /* ------------------------------ */


  /* elementi del frame note */
  /* text */
  GtkWidget *note;
  /* ------------------------------ */


  /* i bottoni principali del widget modulo */
  GtkWidget *ok;
  GtkWidget *cancella;
  GtkWidget *aiuto;  // definito ma da utilizzare in futuro
  /* ------------------------------ */
  
  Info webinfo;
  Info mailinfo;
  Info telinfo;
  
  MODO_MODULO modo;
};


struct _ModuloRubricaClass {
  GnomeDialogClass parent_class;

  void (* modulo_rubrica_aiuto)   (ModuloRubrica *modulo, gint pagina);
};

guint modulo_rubrica_get_type (void);
GtkWidget *modulo_rubrica_new (MODO_MODULO modo);  

void modulo_rubrica_set_modulo(GtkWidget *wid);
GtkWidget *modulo_rubrica_get_modulo(void);
void modulo_rubrica_clean_modulo(ModuloRubrica *m);


/*   funzioni per manipolare contenuto del widget
*/
RubricaItem *modulo_rubrica_estrai_dati(ModuloRubrica *modulo);
void modulo_rubrica_inserisci_dati(ModuloRubrica *m, RubricaItem *r);
void modulo_rubrica_estrai_inserisci_e_chiudi (ModuloRubrica *m);
void modulo_rubrica_estrai_sostituisci_e_chiudi(ModuloRubrica *m);
void modulo_rubrica_modifica_elemento (RubricaItem *rub);
gboolean modulo_rubrica_modulo_is_modificato();

gchar *modulo_rubrica_get_telefono_attivo(void);
void   modulo_rubrica_set_telefono_attivo(gchar *str);
TELEPHONE_TYPE modulo_rubrica_tipo_telefono_int(gchar *tipo);

END_GNOME_DECLS

#endif







