/*
*  Rubrica
*  file: search.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <gnome.h>

#include "rubrica.h"
#include "search.h"
#include "search_int.h"
#include "error.h"


GtkWidget *result_clist;  /* the clist where will show the results */
GList *coda = NULL;       /* results queue    */
WhereSearch where;        /* where search     */
HowSearch how;            /* how search is do */
HowSearch substr;         /* search a string into another string */ 
gchar *who;               /* who/what search */
gchar *str;               
gint trovati;             /* number of item found */ 


void rubrica_search_init()
{
  /* default */
  where   = CARD;               /* search by card  */
  how     = CASE_INSENSITIVE;   /* case insensitive */		 
  substr  = FULL_MATCH;         /* searched string have to match full */
  who     = NULL;
  str     = NULL;
  trovati = 0;
}

void rubrica_search_free_coda(GList *alias)
{     
  g_return_if_fail(alias != NULL);
  
  if (alias->next)
    rubrica_search_free_coda(alias->next);
  else
    g_list_free_1(alias);
}


void rubrica_search_where_set(WhereSearch search_where)
{
  where = (WhereSearch) search_where;
}


/*
 * se hs == CASE_SENSITIVE   la ricerca  effettuata distinguendo 
 *                           tra maiuscole e minuscole
 * se hs == CASE_INSENSITIVE la ricerca  effettuata ignorando i 
 *                           caratteri maiuscoli e minuscoli
 * 
*/
  
void rubrica_search_sensitive_set(HowSearch hs)
{
  how = hs;
}


/*
  se hs == PARTIAL_MATCH cerco la corrispondenza tra alcuni caratteri e le stringhe
  se hs == FULL_MATCH    la ricerca  condotta su stringhe esatte
  
  es: search ipp in pippo (PARTIAL_MATCH)
*/  

void rubrica_search_into_set(HowSearch hs)
{
  substr = hs;
}



void rubrica_search_item()
{
  GList *list;
  GList *ls;
  RubricaItem *item;
  GtkWidget *entry;
  GtkCList *vista_clist;
  gint pos;
  gchar *buffer = NULL;
  
  /* pulisco la coda da una eventuale ricerca precedente */
  if (coda)
    {
      rubrica_search_free_coda(coda);
      coda = NULL;
    }
  
  str = NULL;
  list = rubrica_get_rubrica();

  entry = gnome_entry_gtk_entry(GNOME_ENTRY(rubrica_search_get_search_text()));
 
  result_clist = rubrica_search_get_search_list();
  vista_clist = GTK_CLIST(rubrica_view_get_dati());

  /* get the string we want to search */
  who = gtk_entry_get_text(GTK_ENTRY(entry));

  trovati = 0;
  pos = 0;
  
  /* pulisco la finestra dei risultati della ricerca */
  gtk_clist_clear(GTK_CLIST(result_clist));

  if (who == NULL)
    rubrica_error_no_string();
  else
    while (list)
      {
	gboolean find = FALSE;
	
	item = (RubricaItem*) list->data;
      
	switch(where)     
	  {
	  case CARD:
	    if (item->scheda)
	      {
		str = item->scheda;
	      
	      if ((find = rubrica_search_match(str)))
		rubrica_search_insert_item_found(str, str, pos);
	      }
	    break;

	  case NAME:
	    if (item->nome)
	      {
		str = item->nome;

		if ((find = rubrica_search_match(str)))
		  rubrica_search_insert_item_found(item->scheda, str, pos);
	      }
	    break;
	    
	    case SURNAME:
	      if (item->cognome)
		{
		  str = item->cognome;

		  if ((find = rubrica_search_match(str)))
		    rubrica_search_insert_item_found(item->scheda, str, pos);
  		}
	    break;

	    case STATE:
	      if (item->stato)
		{
		str = item->stato;

		if((find = rubrica_search_match(str)))
		  rubrica_search_insert_item_found(item->scheda, str, pos);
		}
	    break;

	    case CITY:
	      if (item->citta)
		{
		str = item->citta;

		if((find = rubrica_search_match(str)))
		  rubrica_search_insert_item_found(item->scheda, str, pos);
		}
	    break;

	    case PROF:
	      if (item->professione)
		{
		str = item->professione;

		if((find = rubrica_search_match(str)))
		  rubrica_search_insert_item_found(item->scheda, str, pos);
		}
	    break;
	    
	  case WEB:
	    if (item->web_list)
	      {
		ls = item->web_list;
		
		while (ls)
		  {    
		    str = (gchar*) ls->data;		    

		    if ((find = rubrica_search_match(str)))
		      {
			rubrica_search_insert_item_found(item->scheda, str, pos);     
			break;
		      }
		    else
		      ls = g_list_next(ls);		      
		  }
	      }
	    break;
	    
	  case MAIL:
	    if (item->mail_list)
	      {
		ls = item->mail_list;
		
		while (ls)
		  {
		    str = (gchar*) ls->data;  
		    
		    if ((find = rubrica_search_match(str)))
		      {
			rubrica_search_insert_item_found(item->scheda, str, pos);
		  	break;
		      }
		    else
		      ls = g_list_next(ls);		      
		  }
	      }
	    break;
	    
	  case TEL:
	    if (item->tel_list)
	      {
		ls = item->tel_list;
		
		while (ls)
		  {
		    TelNum *tel;
		    
		    tel = ls->data;
		    str = (gchar*) tel->numero_telefono;
		    
		    if ((find = rubrica_search_match(str)))
		      {
			rubrica_search_insert_item_found(item->scheda, str, pos);
			break;
		      }
		    else
		      ls = g_list_next(ls);		      
		  }
	      }
	    break;
	    
	  default:
	    break;
	  }
	
	find = FALSE;

	list = g_list_next(list);
	pos++;
      }

  /* stampo il numero di elementi trovati */
 
  buffer = g_strdup_printf(_(" Item(s) found: %d"), trovati);
  rubrica_search_set_label_result(buffer);
  g_free(buffer);
}

gboolean rubrica_search_match(gchar *str)
{
  gboolean flag;
  
  flag = FALSE;

  /* search is case sensitive */
  if (how == CASE_SENSITIVE)	
    {
      /* search is on a partial string */
      if (substr == PARTIAL_MATCH)
	flag = rubrica_search_submatch(str, how);
      else
	if(strcmp(str, who) == 0)
	  flag = TRUE;
    }
  else   /* search isn't case sensitive */
    {
      if (substr == PARTIAL_MATCH)
	flag = rubrica_search_submatch(str, how);
      else
	if(strcasecmp(str, who) == 0)
	  flag = TRUE;
    }
  return (gboolean) flag;
}

gboolean rubrica_search_submatch(gchar *str, gboolean how)
{
  gboolean find = FALSE;
  gint len = strlen(who);


  if (how == CASE_INSENSITIVE)
    while (*str)
      {	
	if (strncasecmp(str, who, len) == 0)
	  {
	    find = TRUE;
	    break;
	  }	 
	else
	  str++;
      }   
  else   /*  how == CASE_SENSITIVE  */
    while (*str)
      {
	if (strncmp(str, who, len) == 0)
	  {
	    find = TRUE;
	    break;
	  }
	else
	  str++;
      }
  
  return find;
}


void rubrica_search_insert_item_found(gchar *card, gchar *str, gint pos)
{
  gchar *buffer[3];
  gchar *titolo;

  g_return_if_fail(str != NULL);

  coda = g_list_append(coda, (gpointer)pos);

  titolo = NULL;
  if (card)
    buffer[0] = g_strdup(card);
  else 
    buffer[0] = NULL;
  buffer[1] = g_strdup(str);
  buffer[2] = NULL;

  switch (where)
    {
    case NAME:
      titolo = g_strdup(_("First Name"));
      break;

    case SURNAME:
      titolo =  g_strdup(_("Last Name"));
      break;

    case STATE:
      titolo = g_strdup( _("State"));
      break;    

    case CITY:
      titolo = g_strdup(_("City"));
      break;   

    case PROF:
      titolo = g_strdup(_("Profession"));
      break; 
   
    case WEB:
      titolo = g_strdup(_("Url"));
      break;    
      
    case MAIL:
      titolo = g_strdup(_("Email"));
      break;

    case TEL:
      titolo = g_strdup(_("Telephone"));
      break;

    case CARD:
    default:
      break;
    }

  trovati++;
  gtk_clist_set_column_title(GTK_CLIST(result_clist), 1, titolo);
  gtk_clist_append(GTK_CLIST(result_clist), buffer);  
}

void rubrica_search_view_item(gint i)
{
  RubricaItem *r;
  gint pos;

  pos = (gint) g_list_nth_data(coda,i);
  r = rubrica_prendi_elemento(pos);
  rubrica_mostra_scheda(r);  
}



               
